import json

COMMON_TRANSITIONS = {'FADE OUT.', 'CUT TO BLACK.', 'FADE TO BLACK.'}
UPPER_ALPHABETS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ ÄÖÜ'


class FountainElement:
    def __init__(
        self,
        element_type,
        element_text='',
        section_depth=0,
        scene_number='',
        is_centered=False,
        is_dual_dialogue=False,
        original_line=0,
        scene_abbreviation='.',
        original_content=''
    ):
        self.element_type = element_type
        self.element_text = element_text
        self.section_depth = section_depth
        self.scene_number = scene_number
        self.scene_abbreviation = scene_abbreviation
        self.is_centered = is_centered
        self.is_dual_dialogue = is_dual_dialogue
        self.original_line = original_line
        self.original_content = original_content

    def __repr__(self):
        return self.element_type + ': ' + self.element_text

    def to_dict(self):
        return {
            'element_type': self.element_type,
            'element_text': self.element_text,
            'section_depth': self.section_depth,
            'scene_number': self.scene_number,
            'scene_abbreviation': self.scene_abbreviation,
            'is_centered': self.is_centered,
            'is_dual_dialogue': self.is_dual_dialogue,
            'original_line': self.original_line,
            'original_content': self.original_content
        }


class Fountain:
    def __init__(self, string=None, path=None):
        self.metadata = dict()
        self.elements = list()

        if path:
            with open(path) as fp:
                self.contents = fp.read()
        else:
            self.contents = string
        if self.contents != '':
            self.parse()

    def get_elements_as_dict_list(self):
        elements_as_dict = []
        for element in self.elements:
            elements_as_dict.append(element.to_dict())
        return elements_as_dict

    def pretty_print_elements_as_json(self):
        elements_as_dict = self.get_elements_as_dict_list()
        pretty_json = json.dumps(elements_as_dict, indent=4)
        print(pretty_json)

    def parse(self):
        contents = self.contents.strip().replace('\r', '')

        contents_has_metadata = ':' in contents.splitlines()[0]
        contents_has_body = '\n\n' in contents

        if contents_has_metadata and contents_has_body:
            script_head, script_body = contents.split('\n\n', 1)
            self._parse_head(script_head.splitlines())
            self._parse_body(script_body.splitlines())
        elif contents_has_metadata and not contents_has_body:
            self._parse_head(contents.splitlines())
        else:
            self._parse_body(contents.splitlines())

    def _parse_head(self, script_head):
        open_key = None
        for line in script_head:
            line = line.rstrip()
            if line[0].isspace():
                self.metadata[open_key].append(line.strip())
            elif line[-1] == ':':
                open_key = line[0:-1].lower()
                self.metadata[open_key] = list()
            else:
                key, value = line.split(':', 1)
                self.metadata[key.strip().lower()] = [value.strip()]

    def _parse_body(self, script_body):
        is_comment_block = False
        is_inside_dialogue_block = False
        newlines_before = 0
        index = -1
        comment_text = list()

        for linenum, line in enumerate(script_body):
            assert type(line) is str
            index += 1
            line = line.lstrip()
            full_strip = line.strip()

            if (not line or line.isspace()) and not is_comment_block:
                self.elements.append(FountainElement('Empty Line'))
                is_inside_dialogue_block = False
                newlines_before += 1
                continue

            if line.startswith('/*'):
                line = line.rstrip()
                if line.endswith('*/'):
                    text = line.replace('/*', '').replace('*/', '')
                    self.elements.append(
                        FountainElement(
                            'Boneyard',
                            text,
                            original_line=linenum,
                            original_content=line
                        )
                    )
                    is_comment_block = False
                    newlines_before = 0
                else:
                    is_comment_block = True
                    comment_text.append('')
                continue

            if line.rstrip().endswith('*/'):
                text = line.replace('*/', '')
                comment_text.append(text.strip())
                self.elements.append(
                    FountainElement(
                        'Boneyard',
                        '\n'.join(comment_text),
                        original_line=linenum,
                        original_content=line
                    )
                )
                is_comment_block = False
                comment_text = list()
                newlines_before = 0
                continue

            if is_comment_block:
                comment_text.append(line)
                continue

            if line.startswith('==='):
                self.elements.append(
                    FountainElement(
                        'Page Break',
                        line,
                        original_line=linenum,
                        original_content=line
                    )
                )
                newlines_before = 0
                continue

            if len(full_strip) > 0 and full_strip[0] == '=':
                self.elements.append(
                    FountainElement(
                        'Synopsis',
                        full_strip[1:].strip(),
                        original_line=linenum,
                        original_content=line
                    )
                )
                continue

            if (
                newlines_before > 0
                and full_strip.startswith('[[')
                and full_strip.endswith(']]')
            ):
                self.elements.append(
                    FountainElement(
                        'Comment',
                        full_strip.strip('[] \t'),
                        original_line=linenum,
                        original_content=line
                    )
                )
                continue

            if len(full_strip) > 0 and full_strip[0] == '#':
                newlines_before = 0
                depth = full_strip.split()[0].count('#')
                self.elements.append(
                    FountainElement(
                        'Section Heading',
                        full_strip[depth:].strip(),
                        section_depth=depth,
                        original_line=linenum,
                        original_content=line
                    )
                )
                continue

            if len(line) > 1 and line[0] == '.' and line[1] != '.':
                newlines_before = 0
                if full_strip[-1] == '#' and full_strip.count('#') > 1:
                    scene_number_start = len(full_strip) - \
                        full_strip[::-1].find('#', 1) - 1
                    self.elements.append(
                        FountainElement(
                            'Scene Heading',
                            full_strip[1:scene_number_start].strip(),
                            scene_number=full_strip[
                                scene_number_start:
                            ].strip('#').strip(),
                            original_line=linenum,
                            original_content=line
                        )
                    )
                else:
                    self.elements.append(
                        FountainElement(
                            'Scene Heading',
                            full_strip[1:].strip(),
                            original_line=linenum,
                            original_content=line
                        )
                    )
                continue

            if (
                line[0:4].upper() in
                ['INT ', 'INT.', 'EXT ', 'EXT.', 'EST ', 'EST.', 'I/E ', 'I/E.']
                or line[0:8].upper() in ['INT/EXT ', 'INT/EXT.']
                or line[0:9].upper() in ['INT./EXT ', 'INT./EXT.']
            ):
                newlines_before = 0
                scene_name_start = line.find(line.split()[1])
                if full_strip[-1] == '#' and full_strip.count('#') > 1:
                    scene_number_start = len(full_strip) - \
                        full_strip[::-1].find('#', 1) - 1
                    self.elements.append(
                        FountainElement(
                            'Scene Heading',
                            full_strip[
                                scene_name_start:scene_number_start
                            ].strip(),
                            scene_number=full_strip[
                                scene_number_start:
                            ].strip('#').strip(),
                            original_line=linenum,
                            scene_abbreviation=line.split()[0],
                            original_content=line
                        )
                    )
                else:
                    self.elements.append(
                        FountainElement(
                            'Scene Heading',
                            full_strip[scene_name_start:].strip(),
                            original_line=linenum,
                            scene_abbreviation=line.split()[0],
                            original_content=line
                        )
                    )
                continue

            if full_strip.endswith(' TO:'):
                newlines_before = 0
                self.elements.append(
                    FountainElement(
                        'Transition',
                        full_strip,
                        original_line=linenum,
                        original_content=line
                    )
                )
                continue

            if full_strip in COMMON_TRANSITIONS:
                newlines_before = 0
                self.elements.append(
                    FountainElement(
                        'Transition',
                        full_strip,
                        original_line=linenum,
                        original_content=line
                    )
                )
                continue

            if full_strip[0] == '>':
                newlines_before = 0
                if len(full_strip) > 1 and full_strip[-1] == '<':
                    self.elements.append(
                        FountainElement(
                            'Action',
                            full_strip[1:-1].strip(),
                            is_centered=True,
                            original_line=linenum,
                            original_content=line
                        )
                    )
                else:
                    self.elements.append(
                        FountainElement(
                            'Transition',
                            full_strip[1:].strip(),
                            original_line=linenum,
                            original_content=line
                        )
                    )
                continue

            if (
                newlines_before > 0
                and index + 1 < len(script_body)
                and script_body[index + 1]
                and not line[0] in ['[', ']', ',', '(', ')']
                and (all([(c in UPPER_ALPHABETS) for c in full_strip])
                     or full_strip[0] == '@')
            ):
                newlines_before = 0
                if full_strip[-1] == '^':
                    for element in reversed(self.elements):
                        if element.element_type == 'Character':
                            element.is_dual_dialogue = True
                            break
                    self.elements.append(
                        FountainElement(
                            'Character',
                            full_strip.lstrip('@').rstrip('^').strip(),
                            is_dual_dialogue=True,
                            original_line=linenum,
                            original_content=line
                        )
                    )
                    is_inside_dialogue_block = True
                else:
                    self.elements.append(
                        FountainElement(
                            'Character',
                            full_strip.lstrip('@'),
                            original_line=linenum,
                            original_content=line
                        )
                    )
                    is_inside_dialogue_block = True
                continue

            if is_inside_dialogue_block:
                if newlines_before == 0 and full_strip[0] == '(':
                    self.elements.append(
                        FountainElement(
                            'Parenthetical',
                            full_strip,
                            original_line=linenum,
                            original_content=line
                        )
                    )
                else:
                    if self.elements[-1].element_type == 'Dialogue':
                        self.elements[-1].element_text = '\n'.join(
                            [self.elements[-1].element_text, full_strip]
                        )
                    else:
                        self.elements.append(
                            FountainElement(
                                'Dialogue',
                                full_strip,
                                original_line=linenum,
                                original_content=line
                            )
                        )
                continue

            if newlines_before == 0 and len(self.elements) > 0:
                self.elements[-1].element_text = '\n'.join(
                    [self.elements[-1].element_text, full_strip])
                newlines_before = 0
            else:
                self.elements.append(
                    FountainElement(
                        'Action',
                        full_strip,
                        original_line=linenum,
                        original_content=line
                    )
                )
                newlines_before = 0



from ...utils import Job
from . import jtask

description = "Genera una plantilla yaml del proyecto."
configMap = {
    "type": "object",
    "properties": {
        "pname": {
            "type": "string",
            "description": "Nombre del proyecto"
        },
        "useDefault": {
            "type": "string",
            "description": "Nombre de la plantilla por defecto que se empleará"
        }
    },
    "required": ["pname"]
}


class MJob(Job.Job):
    def __init__(self):
        super().__init__(description,
                         configMap)

    def task(self):
        if not "useDefault" in self.params:
            self.params['useDefault'] = 'default'

        jtask.run(self.params)


def get():
    return MJob()



import pyttsx3
import logging
import os
import random
from pedalboard import Pedalboard, Reverb, Distortion, Delay, Gain
from pedalboard.io import AudioFile

# Configura el sistema de registro (logging)
logger = logging.getLogger(__name__)

TMP_OUTWAV = "/tmp"

EXAMPLE_VOICE_CONFIG = {
    "voice": {
        "tts": {
            "voiceid": 14,
            "rate": 200,
            "volume": 1
        },
        "filter": [
            {
                "reverb": {
                    "room_size": 0.5,
                    "damping": 0.5,
                    "wet_level": 0.33,
                    "dry_level": 0.4,
                    "width": 1.0,
                    "freeze_mode": 0.0
                }
            },
            {
                "distortion": {
                    "drive_db": 25
                }
            },
            {
                "delay": {
                    "delay_seconds": 0.5,
                    "feedback": 0.0,
                    "mix": 0.5
                }
            },
            {
                "gain": {
                    "gain_db": 1.0
                }
            }
        ]
    }
}


class CharacterAudio:
    def __init__(self, config=None):
        self.engine = pyttsx3.init()
        self._character_config = config
        self.current_say = None  # Almacena la última salida generada
        self.filters = []

        self._configureTTS(self._character_config)
        self._configureFilters(self._character_config)

    def _configureTTS(self, config):
        if config and "voice" in config:
            voice_config = config["voice"]
            if "tts" in voice_config:
                tts_config = voice_config["tts"]
                for key, value in tts_config.items():
                    if key == "voiceid":
                        voice_id = self.engine.getProperty('voices')[value].id
                        self.engine.setProperty('voice', voice_id)
                        logger.debug(
                            f"Configurando propiedad 'voice' a '{voice_id}'")
                    else:
                        self.engine.setProperty(key, value)
                        logger.debug(
                            f"Configurando propiedad '{key}' a '{value}'")

    def _configureFilters(self, config):
        if config and "voice" in config:
            for filter_item in config.get('voice').get("filter", []):
                for filter_name, filter_params in filter_item.items():
                    if filter_name == "reverb":
                        reverb = Reverb(
                            room_size=filter_params.get("room_size", 0.5),
                            damping=filter_params.get("damping", 0.5),
                            wet_level=filter_params.get("wet_level", 0.33),
                            dry_level=filter_params.get("dry_level", 0.4),
                            width=filter_params.get("width", 1.0),
                            freeze_mode=filter_params.get("freeze_mode", 0.0)
                        )
                        self.filters.append(reverb)
                    elif filter_name == "distortion":
                        distortion = Distortion(
                            drive_db=filter_params.get("drive_db", 25)
                        )
                        self.filters.append(distortion)
                    elif filter_name == "delay":
                        delay = Delay(
                            delay_seconds=filter_params.get(
                                "delay_seconds", 0.5),
                            feedback=filter_params.get("feedback", 0.0),
                            mix=filter_params.get("mix", 0.5)
                        )
                        self.filters.append(delay)
                    elif filter_name == "gain":
                        gain = Gain(
                            gain_db=filter_params.get("gain_db", 1.0)
                        )
                        self.filters.append(gain)

    def ttsay(self, text):
        self.engine.say(text)
        self.engine.runAndWait()

    def ttsay_and_save(self, text, out):
        try:
            self.engine.save_to_file(text, out)
            self.engine.runAndWait()
            logger.debug(f"Salida guardada en '{out}'")
        except Exception as e:
            logger.error(f"Error al guardar la salida en '{out}': {str(e)}")

    def addFilters(self, inputwav, out):
        board = Pedalboard(self.filters)
        with AudioFile(inputwav) as f:
            with AudioFile(out, 'w', f.samplerate, f.num_channels) as o:
                while f.tell() < f.frames:
                    chunk = f.read(f.samplerate)
                    effected = board(chunk, f.samplerate, reset=False)
                    o.write(effected)

    def say(self, text, out="/tmp/tmpcharacter.wav"):
        self.current_say = out

        if self.filters:
            logger.debug(f"Aplicando filtros")
            # Extraer el nombre del archivo del parámetro 'out'
            out_filename = os.path.basename(out).split('.')[0]
            # Generar un valor aleatorio de tres dígitos
            random_number = random.randint(100, 999)
            # Construir el nombre del archivo temporal
            temp_filename = f"{out_filename}-{random_number}.tmp.wav"
            # Unir el directorio base con el nombre del archivo temporal
            full_path = os.path.join(TMP_OUTWAV, temp_filename)
            self.ttsay_and_save(text, full_path)
            self.addFilters(full_path, out)
            os.remove(full_path)
        else:
            logger.debug(f"Sin filtros")
            self.ttsay_and_save(text, out)

        logger.debug(f"Salida de say guardada en '{out}'")



import sqlite3
from ...lib import Fountain


def runloader(fountain_file, pname):

    # Conectar o crear la base de datos
    conn = sqlite3.connect(f"{pname}.cst")
    cursor = conn.cursor()

    # Crear una instancia de Fountain y parsear el guion
    with open(fountain_file, 'r', encoding='utf-8') as fountain_file:
        fountain_text = fountain_file.read()

    fountain = Fountain.Fountain(fountain_text)

    current_scene_number = None  # Para rastrear el número de escena actual

    # Insertar la información del guion en la base de datos
    for scene in fountain.elements:
        if scene.element_type == 'Scene Heading':
            # Insertar información de la escena
            current_scene_number = scene.scene_number
            cursor.execute('''
                INSERT INTO Scene (script_id, scene_number, location, description)
                VALUES (?, ?, ?, ?)
            ''', (1, current_scene_number, scene.element_text, ''))

        elif scene.element_type == 'Character':
            # Obtener el ID del personaje por nombre
            cursor.execute(
                'SELECT character_id FROM Character WHERE name = ?', (scene.element_text,))
            character_row = cursor.fetchone()

            if character_row is not None:
                character_id = character_row[0]
            else:
                # El personaje no se encontró, así que lo insertamos en la base de datos
                cursor.execute('''
                    INSERT INTO Character (script_id, name)
                    VALUES (?, ?)
                ''', (1, scene.element_text))

                # Recuperar el ID del personaje recién insertado
                cursor.execute('SELECT last_insert_rowid()')
                character_id = cursor.fetchone()[0]

        elif scene.element_type == 'Dialogue':
            # Obtener el ID de la última escena insertada
            cursor.execute('SELECT MAX(scene_id) FROM Scene')
            scene_id = cursor.fetchone()[0]

            # Insertar el diálogo
            cursor.execute('''
                INSERT INTO Dialogue (scene_id, character_id, text)
                VALUES (?, ?, ?)
            ''', (scene_id, character_id, scene.element_text))

        elif scene.element_type == 'Element':
            # Obtener el ID de la última escena insertada
            cursor.execute('SELECT MAX(scene_id) FROM Scene')
            scene_id = cursor.fetchone()[0]

            # Insertar elemento en la tabla Element
            cursor.execute('''
                INSERT INTO Element (scene_id, element_type, text)
                VALUES (?, ?, ?)
            ''', (scene_id, scene.element_type, scene.element_text))

    # Guardar los cambios y cerrar la conexión
    conn.commit()
    conn.close()



import importlib
import logging
import traceback

# Configuración básica del logger
logging.basicConfig(level=logging.ERROR)
logger = logging.getLogger(__name__)


class ModuleRunner:
    def __init__(self, job_name):
        self.job_name = job_name
        self.module_name = f"cassette.jobs.job_{job_name.lower()}.{job_name.capitalize()}"
        self.module = None

        try:
            self.module = importlib.import_module(self.module_name)
        except Exception as e:
            traceback.print_exc()
            logger.error(
                f"Error al cargar el módulo {self.module_name}: {str(e)}")

    def run(self, params):
        if self.module is None:
            logger.error(
                f"El módulo {self.module_name} no se pudo cargar. Consulta los registros de errores para más detalles.")
            return None

        if hasattr(self.module, 'get'):
            try:
                return self.module.get().run(params)
            except Exception as e:
                traceback.print_exc()
                logger.error(
                    f"Error al ejecutar el módulo {self.module_name}: {str(e)}")
                return None
        else:
            logger.error(
                f"La función 'get' no está definida en el módulo {self.module_name}.")
            return None

    def describe(self):
        if self.module is None:
            logger.error(
                f"El módulo {self.module_name} no se pudo cargar. Consulta los registros de errores para más detalles.")
            return None

        if hasattr(self.module, 'get'):
            try:
                return self.module.get().description
            except Exception as e:
                logger.error(
                    f"Error al ejecutar la función 'get().description' en el módulo {self.module_name}: {str(e)}")
                return None
        else:
            logger.error(
                f"La función 'get' no está definida en el módulo {self.module_name}.")
            return None



import sqlite3
import yaml
import json


def load_tags(name, yaml_data):
    conn = sqlite3.connect(name)
    cursor = conn.cursor()

    for key, value in yaml_data.get("tags", {}).items():
        cursor.execute(
            "INSERT INTO Tags (name, value) VALUES (?, ?);", (key, value))

    conn.commit()
    conn.close()


def load_characters(name, yaml_data):
    conn = sqlite3.connect(name)
    cursor = conn.cursor()

    for character in yaml_data.get("characters", []):
        character_id = character.get("id")
        tts = json.dumps(character)
        cursor.execute(
            "INSERT INTO CharacterDescriptor (character_id, tts) VALUES (?, ?);", (character_id, tts))

    conn.commit()
    conn.close()


def load_timeline(name, yaml_data):
    conn = sqlite3.connect(name)
    cursor = conn.cursor()

    timeline = yaml_data.get("timeline", [])
    for sequence_id, item in enumerate(timeline):
        sequence_type = item.get("type")
        content = json.dumps(item)
        cursor.execute("INSERT INTO Timeline (sequence_id, type, content) VALUES (?, ?, ?);",
                       (sequence_id, sequence_type, content))

    conn.commit()
    conn.close()


def runloader(timeline_file, pname):

    # Cargar datos desde el archivo YAML
    nombre_base_de_datos = f"{pname}.cst"
    yaml_file = timeline_file

    with open(yaml_file, 'r') as f:
        yaml_data = yaml.safe_load(f)

    # Cargar los datos en las tablas
    load_tags(nombre_base_de_datos, yaml_data)
    load_characters(nombre_base_de_datos, yaml_data)
    load_timeline(nombre_base_de_datos, yaml_data)



from pydub import AudioSegment
import yaml


class VoiceLayer:
    def __init__(self):
        self.sound_layer = AudioSegment.empty()
        self.timeline = []

    def load_timeline(self, timeline_yaml):
        # with open(timeline_yaml, 'r') as file:
        #    self.timeline = yaml.safe_load(file)['timeline']
        self.timeline = timeline_yaml
        print(self.timeline)

    def mix_audio(self):
        for i, item in enumerate(self.timeline):
            if item['type'] == 'sound':
                print("Procesando sonido")
                audio = AudioSegment.from_file(item['track'])

                if 'duration' in item:
                    duration = int(item['duration']) * \
                        1000  # Duración en milisegundos
                    audio = audio[:duration]

                if 'volume' in item:
                    if item['volume'] == 'fadein':
                        audio = audio.fade_in(3000)  # 3 segundos
                    elif item['volume'] == 'fadeout':
                        audio = audio.fade_out(3000)  # 3 segundos
                    elif item['volume'] == 'inout':
                        audio = audio.fade_in(3000).fade_out(
                            3000)  # 3 segundos

                if i > 0 and self.timeline[i - 1]['type'] == 'scene':
                    # Aplicar transición cruzada con la escena anterior
                    crossfade_duration = 1200  # 1.2 seg
                    crossfade = audio[:crossfade_duration].overlay(
                        self.sound_layer[-crossfade_duration:], loop=True)
                    self.sound_layer = self.sound_layer[:-crossfade_duration] + \
                        crossfade + audio[crossfade_duration:]
                else:
                    self.sound_layer += audio

            elif item['type'] == 'scene':
                print("Procesando escena")
                scene_audio = AudioSegment.from_file(item['scene_id'])
                if 'background' in item:

                    background_audio = AudioSegment.from_file(
                        item['background'])

                    # Ajustar el volumen del fondo
                    background_audio = background_audio - 20

                    scene_audio = scene_audio.overlay(
                        background_audio, loop=True)

                if i > 0 and self.timeline[i - 1]['type'] == 'sound':
                    # Aplicar transición cruzada con el sonido anterior
                    crossfade_duration = 1500  # 3 segundos
                    crossfade = scene_audio[:crossfade_duration].overlay(
                        self.sound_layer[-crossfade_duration:], loop=True)
                    self.sound_layer = self.sound_layer[:-crossfade_duration] + \
                        crossfade + scene_audio[crossfade_duration:]

    def export_mixed_audio(self, output_path):
        self.sound_layer.export(output_path, format="mp3")


"""
if __name__ == "__main__":
    mixer = VoiceLayer()
    mixer.load_timeline("timeline.yaml")
    mixer.mix_audio()
    mixer.export_mixed_audio("output.mp3")
"""



import os
import sqlite3
import logging
from ...lib.character import CharacterAudioRender


def run(params):
    logger = logging.getLogger(__name__)

    # 1. Mira si existe el directorio indicado en `params['storepath']`.
    # Si no existe, créalo y notifica con un logger.debug que lo ha creado.
    storepath = params['storepath']
    if not os.path.exists(storepath):
        os.makedirs(storepath)
        logger.debug(f"Created directory: {storepath}")

    # 2. Dentro de este directorio crea una carpeta llamada `characters`
    characters_dir = os.path.join(storepath, "characters")
    if not os.path.exists(characters_dir):
        os.makedirs(characters_dir)

    # 3. Se conecta a la sqlite3 especificada en `params['pname']`.
    db_file = f"{params['pname']}.cst"
    if not os.path.exists(db_file):
        logger.error(f"The project {db_file} does not exist.")
        return

    conn = sqlite3.connect(db_file)
    cursor = conn.cursor()

    # 4. Obtiene el `character_id` de todos los personajes.
    cursor.execute('SELECT character_id FROM Character;')
    character_ids = cursor.fetchall()
    conn.close()

    for character_id in character_ids:
        character_id = character_id[0]
        output_dir = os.path.join(characters_dir, str(character_id))
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        renderer = CharacterAudioRender.CharacterAudioRender(
            character_id, db_file)
        renderer.renderAllDialogs(output_dir)


"""
if __name__ == "__main__":
    # Example usage:
    params = {
        'storepath': "/path/to/store",
        'pname': "/path/to/project.db"
    }
    run(params)
"""



import os
import sqlite3


class DatabaseAlreadyExistsError(Exception):
    pass


def createDB(name):
    if os.path.exists(f"{name}.cst"):
        raise DatabaseAlreadyExistsError(f"El archivo {name}.cst ya existe.")

    conn = sqlite3.connect(f"{name}.cst")
    # Crear un cursor para ejecutar comandos SQL
    cursor = conn.cursor()

    # Sentencia SQL para crear la tabla Script
    cursor.execute('''
    CREATE TABLE IF NOT EXISTS Script (
        script_id INTEGER PRIMARY KEY,
        title TEXT,
        author TEXT,
        date_created DATE,
        description TEXT
    );
    ''')

    # Sentencia SQL para crear la tabla Scene
    cursor.execute('''
    CREATE TABLE IF NOT EXISTS Scene (
        scene_id INTEGER PRIMARY KEY,
        script_id INTEGER,
        scene_number TEXT,
        location TEXT,
        audioPath   TEXT,
        description TEXT,
        FOREIGN KEY (script_id) REFERENCES Script(script_id)
    );
    ''')

    # Sentencia SQL para crear la tabla Character
    cursor.execute('''
    CREATE TABLE IF NOT EXISTS Character (
        character_id INTEGER PRIMARY KEY,
        script_id INTEGER,
        name TEXT,
        FOREIGN KEY (script_id) REFERENCES Script(script_id)
    );
    ''')

    # Sentencia SQL para crear la tabla Dialogue
    cursor.execute('''
    CREATE TABLE IF NOT EXISTS Dialogue (
        dialogue_id INTEGER PRIMARY KEY,
        scene_id INTEGER,
        character_id INTEGER,
        text TEXT,
        audioPath TEXT,
        FOREIGN KEY (scene_id) REFERENCES Scene(scene_id),
        FOREIGN KEY (character_id) REFERENCES Character(character_id)
    );
    ''')

    # Sentencia SQL para crear la tabla Element
    cursor.execute('''
    CREATE TABLE IF NOT EXISTS Element (
        element_id INTEGER PRIMARY KEY,
        scene_id INTEGER,
        element_type TEXT,
        text TEXT,
        FOREIGN KEY (scene_id) REFERENCES Scene(scene_id)
    );
    ''')

    # Guardar los cambios y cerrar la conexión
    conn.commit()
    conn.close()



import os
import sqlite3
import logging
import json
from ...lib.mixer import AudioMixer


def extract_content(dictionary_list):
    content_list = []
    for item in dictionary_list:
        if 'content' in item:
            content_list.append(item['content'])
    return content_list


def run(params):
    logger = logging.getLogger(__name__)

    # 1. Mira si existe el directorio indicado en `params['storepath']`.
    # Si no existe, créalo y notifica con un logger.debug que lo ha creado.
    storepath = params['storepath']
    if not os.path.exists(storepath):
        os.makedirs(storepath)
        logger.debug(f"Created directory: {storepath}")

    # 2. Dentro de este directorio crea una carpeta llamada `film`
    film_dir = os.path.join(storepath, "film")
    if not os.path.exists(film_dir):
        os.makedirs(film_dir)

    # 3. Se conecta a la sqlite3 especificada en `params['pname']`.
    db_file = f"{params['pname']}.cst"
    if not os.path.exists(db_file):
        logger.error(f"The project {db_file} does not exist.")
        return

    conn = sqlite3.connect(db_file)
    cursor = conn.cursor()

    # 4. Recupera el contenido de la tabla Tags y monta un diccionario name:value
    tags_dict = {}
    cursor.execute("SELECT name, value FROM Tags")
    for row in cursor.fetchall():
        name, value = row
        tags_dict[name] = value

    # 5. Recupera el contenido de la tabla Timeline
    timeline_entries = []
    cursor.execute("SELECT type, content FROM Timeline")
    for row in cursor.fetchall():
        entry_type, content_json = row
        content_dict = json.loads(content_json)
        timeline_entries.append({"type": entry_type, "content": content_dict})

    # 6. Itera sobre los elementos del array y realiza las sustituciones
    for entry in timeline_entries:
        if entry["type"] == "scene":
            scene_id = entry["content"]["scene_id"]
            cursor.execute(
                "SELECT audioPath FROM Scene WHERE scene_id=?", (scene_id,))
            audio_path = cursor.fetchone()
            if audio_path:
                entry["content"]["scene_id"] = audio_path[0]
            if 'background' in entry["content"]:
                entry["content"]["background"] = tags_dict[entry["content"]["background"]]
        elif entry["type"] == "sound":
            track = entry["content"].get("track")
            if track in tags_dict:
                entry["content"]["track"] = tags_dict[track]

    mixer = AudioMixer.VoiceLayer()
    mixer.load_timeline(extract_content(timeline_entries))
    mixer.mix_audio()
    mixer.export_mixed_audio(film_dir + "/" + params['pname'] + ".mp3")


"""
if __name__ == "__main__":
    # Example usage:
    params = {
        'storepath': "/path/to/store",
        'pname': "/path/to/project.db"
    }
    run(params)
"""



import click
import os
from ..utils.EnvConfig import EnvConfig
from ..utils.ModuleRunner import ModuleRunner


@click.command()
def list_jobs():
    """
    Lista las carpetas bajo el directorio CPOC_JOBS_PATH que comienzan con 'job_'.
    """
    # Obtén la variable de entorno CPOC_JOBS_PATH
    jobs_path = EnvConfig.get_variable('CPOC_JOBS_PATH')

    click.echo(f'Listando trabajos en: {jobs_path}')
    job_folders = [folder for folder in os.listdir(
        jobs_path) if folder.startswith('job_')]

    if job_folders:
        # Quita el prefijo 'job_' de los nombres de las carpetas
        job_names = [folder[len('job_'):] for folder in job_folders]
        click.echo('Trabajos disponibles:')
        for job_name in job_names:
            description = ModuleRunner(job_name).describe()
            click.echo(f'- {job_name}: {description}')
    else:
        click.echo('No se encontraron trabajos.')


def register_commands(cli):
    cli.add_command(list_jobs)


@click.group()
def cli():
    """
    Mi aplicación de línea de comandos.
    """
    pass


# Registra el comando 'run' en la aplicación principal.
register_commands(cli)


if __name__ == '__main__':
    cli()



from setuptools import setup, find_packages
import os

with open("README.md", "r", encoding="utf-8") as readme_file:
    long_description = readme_file.read()


def getRequirements():
    with open("requirements.txt", "r", encoding="utf-8") as requirements_file:
        install_requires = [line.strip()
                            for line in requirements_file if line.strip()]


setup(
    name="alpezacassette",
    version="0.1.4",
    description="Renderizador de dialogos .fountain a audio",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="alpeza",
    packages=find_packages(),
    include_package_data=True,
    install_requires=getRequirements(),
    url="https://github.com/alpeza/cassette",
    entry_points={
        "console_scripts": [
            "cassette = cassette.main:main",
        ],
    },
    package_data={
        "": ["utils/.env"],
    },
)



from ...lib import Descriptor
import json


def formato_minutos(minutos):
    sec = minutos * 60
    sec = sec % (24 * 3600)
    hour = sec // 3600
    sec %= 3600
    min = sec // 60
    sec %= 60
    return "%02d:%02d:%02d" % (hour, min, sec)


def print_ascii_box(text):
    box_width = len(text) + 4
    horizontal_line = "+" + "-" * box_width + "+"
    empty_line = "| " + " " * len(text) + "   |"

    print(horizontal_line)
    print(empty_line)
    print(f"| {text}  |")
    print(empty_line)
    print(horizontal_line)


def print_readable_info(script_info):
    print_ascii_box("📊 Estadísticas:")
    print(
        f"Total de Caracteres Distintos: {script_info['stats']['totalCaracteres']}")
    print(f"Total de Escenas: {script_info['stats']['totalScenes']}")
    print(
        f"Total de Líneas de Diálogo: {script_info['stats']['totalLineasDialogo']}")
    print(
        f"Total de Palabras en Diálogo: {script_info['stats']['totalDePalabras']}")
    print(
        f"Minutos estimados de Diálogo: {formato_minutos(script_info['stats']['estimacionTiempo'])}\n")

    print_ascii_box("🏠 Escenas:")
    for scene in script_info['scenes']:
        print(
            f"{scene['pos']}, {formato_minutos(scene['time'])}, {scene['name']}")

    print_ascii_box("👨‍🦰 Caracteres: ")
    for character in script_info['caracteres']:
        print(character)


def print_json_info(script_info):
    json_info = json.dumps(script_info, indent=4)
    print(json_info)


def run(params):
    dsc = Descriptor.Descriptor(f"{params['pname']}.cst").get_script_info()
    if "json" in params:
        print_json_info(dsc)
    else:
        print_readable_info(dsc)
